# -*- coding: utf-8 -*-
from Acquisition import aq_base, aq_inner
from collective.contentrules.mailfromfield import logger
from collective.contentrules.mailfromfield import messageFactory as _
from email.message import EmailMessage
from OFS.SimpleItem import SimpleItem
from plone import api
from plone.app.contentrules.actions.mail import MailAddForm, MailEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData
from plone.registry.interfaces import IRegistry
from plone.stringinterp.interfaces import IStringInterpolator
from Products.CMFPlone.utils import safe_unicode
from six.moves import filter
from zope import schema
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implementer
from zope.interface import Interface
from zope.interface.interfaces import ComponentLookupError

import six


class IMailFromFieldAction(Interface):
    """Definition of the configuration available for a mail action"""

    subject = schema.TextLine(
        title=_("Subject"),
        description=_("Subject of the message"),
        required=True,
    )

    source = schema.TextLine(
        title=_("Sender email"),
        description=_(
            "The email address that sends the email. If no email is "
            "provided here, it will use the portal from address."
        ),
        required=False,
    )

    fieldName = schema.TextLine(
        title=_("Source field"),
        description=_(
            "Put there the field name from which get the e-mail. "
            "You can provide an attribute name, a method name, an AT field name or "
            "ZMI property"
        ),
        required=True,
    )

    target = schema.Choice(
        required=True,
        title=_("Target element"),
        description=_(
            "help_target",
            default=(
                "Choose to get the address info from: the container "
                "where the rule is activated on, the content who triggered "
                "the event or the parent of the triggering content."
            ),
        ),
        default="object",
        vocabulary="collective.contentrules.mailfromfield.vocabulary.targetElements",
    )

    message = schema.Text(
        title=_("Mail message"),
        description=_(
            "help_message",
            default="Type in here the message that you want to mail. You can "
            "use some dynamic strings that will be replaced with relative "
            "values. See Substitutions table to see all available options.",
        ),
        required=True,
    )


@implementer(IMailFromFieldAction, IRuleElementData)
class MailFromFieldAction(SimpleItem):
    """
    The implementation of the action defined before
    """

    subject = ""
    source = ""
    fieldName = ""
    target = ""
    message = ""

    element = "plone.actions.MailFromField"

    @property
    def summary(self):
        return _(
            "action_summary",
            default='Email to users defined in the "${fieldName}" data',
            mapping=dict(fieldName=self.fieldName),
        )


@implementer(IExecutable)
@adapter(Interface, IMailFromFieldAction, Interface)
class MailActionExecutor(object):
    """The executor for this action."""

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event
        self.portal = self.get_portal()
        self.mapping = self.get_mapping()

    def get_portal(self):
        """Get's the portal object"""
        urltool = api.portal.get_tool("portal_url")
        return urltool.getPortalObject()

    def get_mapping(self):
        """Return a mapping that will replace markers in the template"""
        obj_title = safe_unicode(self.event.object.Title())  # NOQA
        event_url = self.event.object.absolute_url()  # NOQA
        section_title = safe_unicode(self.context.Title())
        section_url = self.context.absolute_url()
        return {
            # "url": event_url,
            # "title": obj_title,
            "section_name": section_title,
            "section_url": section_url,
        }

    def expand_markers(self, text):
        """Replace markers in text with the values in the mapping"""
        for key, value in six.iteritems(self.mapping):
            if not isinstance(value, six.text_type):
                value = value.decode("utf-8")
            text = text.replace("${%s}" % key, value)
        return text

    def get_from(self):
        """Get the from address"""
        source = self.element.source
        if source:
            return source

        # no source provided, looking for the site wide "from" email address
        from_address = None
        registry = getUtility(IRegistry)
        record = registry.records.get("plone.email_from_address", None)
        if record:
            from_address = record.value

        if not from_address:
            raise ValueError(
                "You must provide a source address for this "
                "action or enter an email in the portal "
                "properties"
            )

        from_name = ""
        record_name = registry.records.get("plone.email_from_name", None)
        if record_name:
            from_name = record_name.value
        source = ("%s <%s>" % (from_name, from_address)).strip()
        return source

    def get_target_obj(self):
        """Get's the target object, i.e. the object that will provide the field
        with the email address
        """
        target = self.element.target
        if target == "object":
            obj = self.context
        elif target == "parent":
            obj = self.event.object.aq_parent
        elif target == "target":
            obj = self.event.object
        else:
            raise ValueError(target)
        return aq_base(aq_inner(obj))

    def get_recipients(self):
        """
        The recipients of this mail
        """
        # Try to load data from the target object
        fieldName = str(self.element.fieldName)
        obj = self.get_target_obj()
        recipients = None

        # 1: object attribute
        try:
            attr = obj.__getattribute__(fieldName)
            # 3: object method
            if hasattr(attr, "__call__"):
                recipients = attr()
                logger.debug("getting e-mail from %s method" % fieldName)
            else:
                recipients = attr
                logger.debug("getting e-mail from %s attribute" % fieldName)
        except AttributeError:
            # 2: try with AT field
            # if IBaseContent.providedBy(obj):
            #     field = obj.getField(fieldName)
            #     if field:
            #         recipients = field.get(obj)
            #     else:
            #         recipients = False
            # else:
            #     recipients = False
            # if not recipients:
            #     recipients = obj.getProperty(fieldName, [])
            #     if recipients:
            #         logger.debug('getting e-mail from %s CMF property'
            #                      % fieldName)
            # else:
            #     logger.debug('getting e-mail from %s AT field' % fieldName)
            pass

        # maybe for some reason we could execute this action without any
        # recipients. It can't be that the page breaks due to this kind
        # of problem
        if not recipients:
            return []

        # now transform recipients in a iterator, if needed
        if type(recipients) == str or type(recipients) == six.text_type:
            recipients = [str(recipients)]
        return list(filter(bool, recipients))

    def get_mailhost(self):
        """
        The recipients of this mail
        """
        mailhost = api.portal.get_tool("MailHost")
        if not mailhost:
            error = "You must have a Mailhost utility to execute this action"
            raise ComponentLookupError(error)
        return mailhost

    def __call__(self):
        """
        Does send the mail
        """
        mailhost = self.get_mailhost()
        source = self.get_from()
        recipients = self.get_recipients()

        obj = self.event.object

        interpolator = IStringInterpolator(obj)
        subject = self.element.subject
        message = self.element.message
        # Section title/url
        subject = self.expand_markers(subject)
        message = self.expand_markers(message)
        # All other stringinterp
        subject = interpolator(subject).strip()
        message = interpolator(message).strip()

        email_charset = None
        registry = getUtility(IRegistry)
        record = registry.records.get("plone.email_charset", None)
        if record:
            email_charset = record.value

        msg = EmailMessage()
        msg.set_content(message, charset=email_charset)
        msg["Subject"] = subject
        msg["From"] = source
        msg["To"] = ""

        self.manage_attachments(msg=msg)
        for email_recipient in recipients:
            msg.replace_header("To", email_recipient)
            # we set immediate=True because we need to catch exceptions.
            # by default (False) exceptions are handled by MailHost and we can't catch them.
            mailhost.send(msg, charset=email_charset, immediate=True)

            logger.debug("sending to: %s" % email_recipient)
        return True

    def manage_attachments(self, msg):
        """
        Customize this when needed
        """
        pass


class MailFromFieldAddForm(MailAddForm):
    schema = IMailFromFieldAction
    Type = MailFromFieldAction


class MailFromFieldAddFormView(ContentRuleFormWrapper):
    form = MailFromFieldAddForm


class MailFromFieldEditForm(MailEditForm):
    schema = IMailFromFieldAction


class MailFromFieldEditFormView(ContentRuleFormWrapper):
    form = MailFromFieldEditForm



# -*- coding: UTF-8 -*-
from collective.contentrules.mailfromfield.actions.mail import (
    MailFromFieldAction,
    MailFromFieldAddForm,
    MailFromFieldEditForm,
)
from collective.contentrules.mailfromfield.testing import (
    CONTENTRULES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.contentrules.rule import Rule
from plone.app.contentrules.tests.base import ContentRulesTestCase
from plone.app.testing import SITE_OWNER_NAME
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IExecutable, IRuleAction
from plone.testing.zope import login
from zope.component import getMultiAdapter, getUtility
from zope.interface.interfaces import IObjectEvent
from zope.interface import implementer


# basic test structure copied from plone.app.contentrules test_action_mail.py


@implementer(IObjectEvent)
class DummyEvent(object):
    def __init__(self, object):
        self.object = object


class TestMailAction(ContentRulesTestCase):
    layer = CONTENTRULES_INTEGRATION_TESTING
    maxDiff = None

    def setUp(self):
        super(TestMailAction, self).setUp()
        self.app = self.layer["app"]
        self.mailhost = self.layer["portal"].MailHost
        self.folder.setTitle("Càrtella")
        self.folder.d1.setTitle("Dòcumento")

    # BBB
    def loginAsPortalOwner(self, userName=SITE_OWNER_NAME):
        """Log in to the portal as the user who created it."""
        login(self.app["acl_users"], userName)

    def testRegistered(self):
        element = getUtility(IRuleAction, name="plone.actions.MailFromField")
        self.assertEquals("plone.actions.MailFromField", element.addview)
        self.assertEquals("edit", element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name="plone.actions.MailFromField")
        storage = getUtility(IRuleStorage)
        storage["foo"] = Rule()
        rule = self.portal.restrictedTraverse("++rule++foo")

        adding = getMultiAdapter((rule, self.portal.REQUEST), name="+action")
        addview = getMultiAdapter((adding, self.portal.REQUEST), name=element.addview)
        self.failUnless(isinstance(addview.form_instance, MailFromFieldAddForm))

        addview.form_instance.createAndAdd(
            data={
                "subject": "My Subject",
                "source": "foo@bar.be",
                "fieldName": "foo",
                "target": "object",
                "message": "Hey, Oh!",
            }
        )

        e = rule.actions[0]
        self.failUnless(isinstance(e, MailFromFieldAction))
        # TODO
        # self.assertEquals("My Subject", e.subject)
        # self.assertEquals("foo@bar.be", e.source)
        # self.assertEquals("foo", e.fieldName)
        # self.assertEquals("object", e.target)
        # self.assertEquals("Hey, Oh!", e.message)

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name="plone.actions.MailFromField")
        e = MailFromFieldAction()
        editview = getMultiAdapter((e, self.folder.REQUEST), name=element.editview)
        self.failUnless(isinstance(editview.form_instance, MailFromFieldEditForm))

    def testExecuteNoSource(self):
        self.loginAsPortalOwner()
        e = MailFromFieldAction()
        e.message = "Document created !"
        e.fieldName = "foo_attr"
        e.target = "object"
        e.subject = "Subject"
        self.folder.foo_attr = "member1@dummy.org"
        email_from_address = api.portal.get_registry_record("plone.email_from_address")
        email_from_name = api.portal.get_registry_record("plone.email_from_name")
        api.portal.set_registry_record("plone.email_from_address", "")
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        self.assertRaises(ValueError, ex)
        # if we provide a site mail address this won't fail anymore
        api.portal.set_registry_record("plone.email_from_address", "manager@portal.be")
        api.portal.set_registry_record("plone.email_from_name", "The Big Boss")
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: The Big Boss <manager@portal.be>", mailSent)
        self.assertIn(b"Document created !", mailSent)

        api.portal.set_registry_record("plone.email_from_address", email_from_address)
        api.portal.set_registry_record("plone.email_from_name", email_from_name)

    def testExecuteSimpleByAttribute(self):
        self.loginAsPortalOwner()
        self.folder.foo_attr = "member1@dummy.org"
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "foo_attr"
        e.target = "object"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"'D=C3=B2cumento' created in http://nohost/plone/f1/d1 - Sec=\ntion is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteTargetByAttribute(self):
        self.loginAsPortalOwner()
        self.folder.d1.foo_attr = "member1@dummy.org"
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "foo_attr"
        e.target = "target"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"'D=C3=B2cumento' created in http://nohost/plone/f1/d1 - Sec=\ntion is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteSimpleByMethod(self):
        self.loginAsPortalOwner()
        self.folder.setDescription("member1@dummy.org")
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "Description"
        e.target = "object"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"'D=C3=B2cumento' created in http://nohost/plone/f1/d1 - Sec=\ntion is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteTargetByFieldName(self):
        self.loginAsPortalOwner()
        self.folder.d1.text = "member1@dummy.org"
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "text"
        e.target = "target"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"'D=C3=B2cumento' created in http://nohost/plone/f1/d1 - Sec=\ntion is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteSimpleByCMFProperty(self):
        self.loginAsPortalOwner()
        self.folder.manage_addProperty("foo_property", "member1@dummy.org", "string")
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "foo_property"
        e.target = "object"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"'D=C3=B2cumento' created in http://nohost/plone/f1/d1 - Sec=\ntion is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteFolderModify(self):
        # can happen as rules are not triggered on the rule root itself
        self.loginAsPortalOwner()
        self.folder.foo_property = "member1@dummy.org"
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "foo_property"
        e.target = "object"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder)), IExecutable)
        ex()
        mailSent = self.mailhost.messages[0]
        self.assertIn(b'Content-Type: text/plain; charset="utf-8"', mailSent)
        self.assertIn(b"To: member1@dummy.org", mailSent)
        self.assertIn(b"From: foo@bar.be", mailSent)
        self.assertIn(
            b"C=C3=A0rtella' created in http://nohost/plone/f1 - Section=\n is 'C=C3=A0rtella' (http://nohost/plone/f1) !",
            mailSent,
        )

    def testExecuteEmptyValue(self):
        self.loginAsPortalOwner()
        self.folder.foo_attr = ""
        e = MailFromFieldAction()
        e.source = "foo@bar.be"
        e.fieldName = "foo_attr"
        e.target = "object"
        e.subject = "Subject"
        e.message = "Còntènt '${title}' created in ${url} - Section is '${section_name}' (${section_url}) !"
        getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)), IExecutable)()
        self.assertEqual(self.mailhost.messages, [])


def test_suite():
    from unittest import makeSuite, TestSuite

    suite = TestSuite()
    suite.addTest(makeSuite(TestMailAction))
    return suite



# -*- coding: utf-8 -*-

try:
    from zope.app.schema.vocabulary import IVocabularyFactory
except ImportError:
    # Plone 4.1
    from zope.schema.interfaces import IVocabularyFactory

from collective.contentrules.mailfromfield import messageFactory as _
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary


@implementer(IVocabularyFactory)
class TargetElementsVocabulary(object):
    """Vocabulary factory for possible rule target"""

    def __call__(self, context):
        return SimpleVocabulary(
            [
                SimpleTerm("object", "object", _("From rule's container")),
                SimpleTerm(
                    "target",
                    "target",
                    _("From content that triggered the event"),
                ),
                SimpleTerm("parent", "parent", _("From content's parent")),
            ]
        )


targetElements = TargetElementsVocabulary()



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import (
    FunctionalTesting,
    IntegrationTesting,
    MOCK_MAILHOST_FIXTURE,
    PloneSandboxLayer,
)

import collective.contentrules
import plone.app.contentrules


class CollectiveContentrulesMailfromfieldLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML("testing.zcml", package=plone.app.contentrules.tests)
        self.loadZCML(package=collective.contentrules.mailfromfield)


CONTENTRULES_FIXTURE = CollectiveContentrulesMailfromfieldLayer()


CONTENTRULES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CONTENTRULES_FIXTURE, MOCK_MAILHOST_FIXTURE),
    name="CollectiveContentrulesMailfromfieldLayer:IntegrationTesting",
)


CONTENTRULES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(CONTENTRULES_FIXTURE,),
    name="CollectiveContentrulesMailfromfieldLayer:FunctionalTesting",
)



# -*- coding: utf-8 -*-
from logging import getLogger
from zope.i18nmessageid import MessageFactory


logger = getLogger("collective.contentrules.mailfromfield")
messageFactory = MessageFactory("collective.contentrules.mailfromfield")


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from setuptools import setup, find_packages
import os

version = "1.2.0"

tests_require = ["plone.app.robotframework"]

setup(
    name="collective.contentrules.mailfromfield",
    version=version,
    description="A Plone content rule for send e-mail to addresses taken from the content",
    long_description=open("README.rst").read()
    + "\n"
    + open(os.path.join("docs", "HISTORY.rst")).read(),
    # Get more strings from
    # http://pypi.python.org/pypi?:action=list_classifiers
    classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Communications :: Email",
        "License :: OSI Approved :: GNU General Public License (GPL)",
    ],
    keywords="plone rules mail plonegov",
    author="RedTurtle Technology",
    author_email="sviluppoplone@redturtle.it",
    project_urls={
        "PyPI": "https://pypi.python.org/pypi/collective.contentrules.mailfromfield",
        "Source": "https://github.com/RedTurtle/collective.contentrules.mailfromfield",
        "Tracker": "https://github.com/RedTurtle/collective.contentrules.mailfromfield/issues",
    },
    license="GPL",
    packages=find_packages(exclude=["ez_setup"]),
    namespace_packages=["collective", "collective.contentrules"],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*",
    include_package_data=True,
    zip_safe=False,
    tests_require=tests_require,
    extras_require=dict(test=tests_require),
    install_requires=[
        "setuptools",
        "plone.contentrules",
        # -*- Extra requirements: -*-
    ],
    entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
)
